﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Services.Hive.Common.ServiceContracts;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using HeuristicLab.Services.Hive.Common;
using HeuristicLab.PluginInfrastructure.Manager;
using System.IO;
using System.Reflection;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Clients.Hive.Slave.Tests {
  internal class MockHiveService : IHiveService {
    private static List<Job> jobs;
    private static List<JobData> jobDatas;
    private static List<HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment> hiveExperiments;    
    private static List<Plugin> plugins = null;
    private static IEnumerable<PluginDescription> pDescs = null;
    private static int cnt = 0;

    public List<List<MessageContainer>> Messages { get; set; }
    public static string ServerPluginCachePath { get; set; }
    public static string ServerConfigFilePath { get; set; }

    static MockHiveService() {
      ServerPluginCachePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "PluginCacheServer");
      ServerConfigFilePath = Path.Combine(ServerPluginCachePath, "HeuristicLab 3.3.exe.config");

      byte[] data = PersistenceUtil.Serialize(new MockJob(4000));
      if (plugins == null)
        plugins = ReadPluginsFromServerCache();

      jobs = new List<Job>();
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Waiting, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 0 });
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Offline, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 100, ParentJobId = jobs.First().Id });
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Offline, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 1024, ParentJobId = jobs.First().Id });
      jobs.Add(new Job { Id = Guid.NewGuid(), JobState = JobState.Offline, DateCreated = DateTime.Now, CoresNeeded = 1, MemoryNeeded = 4096, ParentJobId = jobs.First().Id });

      jobDatas = new List<JobData>();
      foreach (var job in jobs) {
        job.PluginsNeededIds = new List<Guid>();
        job.PluginsNeededIds.AddRange(plugins.Select(a => a.Id));
        jobDatas.Add(new JobData() { JobId = job.Id, Data = data });        
      }
      
      hiveExperiments = new List<HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment>();
      hiveExperiments.Add(new HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment() { Id = Guid.NewGuid(), Name = "Hive Exp 1", Description = "", RootJobId = jobs[0].Id });
    }

    private static List<Plugin> ReadPluginsFromServerCache() {
      PluginManager pm = new PluginManager(Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "PluginCacheServer"));
      pm.DiscoverAndCheckPlugins();
      pDescs = pm.Plugins;
      plugins = new List<Plugin>();
      foreach (PluginDescription d in pDescs) {
        Plugin p = new Plugin();
        p.Name = d.Name;
        p.Version = d.Version;
        p.Id = Guid.NewGuid();
        plugins.Add(p);
      }
      return plugins;
    }

    #region Job Methods
    public Guid AddJob(Job job, JobData jobData) {
      job.Id = Guid.NewGuid();
      jobData.JobId = job.Id;
      jobs.Add(job);
      jobDatas.Add(jobData);
      return job.Id;
    }

    public Guid AddChildJob(Guid parentJobId, Job job, JobData jobData) {
      job.Id = Guid.NewGuid();
      job.ParentJobId = parentJobId;
      jobData.JobId = job.Id;
      jobs.Add(job);
      jobDatas.Add(jobData);
      return job.Id;
    }

    public PluginData GetConfigurationFile() {
      byte[] cf = File.ReadAllBytes(ServerConfigFilePath);
      PluginData pd = new PluginData();
      pd.Data = cf;
      return pd;
    }

    public Job GetJob(Guid jobId) {
      return jobs.Where(j => j.Id == jobId).SingleOrDefault();
    }

    public IEnumerable<Job> GetJobs() {
      return jobs;
    }

    public IEnumerable<LightweightJob> GetLightweightJobs(IEnumerable<Guid> jobIds) {
      return jobs.Select(j => new LightweightJob(j)); // not real
    }

    public IEnumerable<LightweightJob> GetLightweightChildJobs(Guid? parentJobId, bool recursive, bool includeParent) {
      return jobs.Where(j => j.ParentJobId.HasValue).Select(j => new LightweightJob(j));
    }

    public JobData GetJobData(Guid jobId) {
      return jobDatas.Where(jd => jd.JobId == jobId).SingleOrDefault();
    }

    public void UpdateJob(Job jobDto, JobData jobDataDto) {
      Console.WriteLine("Update Job called!");
    }

    public void DeleteChildJobs(Guid parentJobId) {
      // do nothing
    }
    #endregion

    #region JobControl Methods
    public Job AquireJob(Guid slaveId) {
      var job = jobs.First();
      job.SlaveId = slaveId;
      return job;
    }

    public void StopJob(Guid jobId) {
      // do nothing
    }

    public void PauseJob(Guid jobId) {
      // do nothing
    }
    #endregion

    #region HiveExperiment Methods
    public HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment GetHiveExperiment(Guid id) {
      return hiveExperiments.Where(he => he.Id == id).SingleOrDefault();
    }

    public IEnumerable<HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment> GetHiveExperiments() {
      return hiveExperiments;
    }

    public Guid AddHiveExperiment(HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment hiveExperimentDto) {
      hiveExperimentDto.Id = Guid.NewGuid();
      hiveExperiments.Add(hiveExperimentDto);
      return hiveExperimentDto.Id;
    }

    public void UpdateHiveExperiment(HeuristicLab.Services.Hive.Common.DataTransfer.HiveExperiment hiveExperimentDto) {
      // do nothing
    }

    public void DeleteHiveExperiment(Guid hiveExperimentId) {
      // do nothing
    }
    #endregion

    #region Login Methods
    public void Hello(Guid slaveId, string name, int cores, int memory) {
      // do nothing
    }

    public void GoodBye() {
      // do nothing
    }    
    
    public List<MessageContainer> Heartbeat(Heartbeat heartbeat) {
      if (Messages != null && cnt < Messages.Count) {
        return Messages[cnt++];
      } else {
        return new List<MessageContainer>();
      }
    }
      
    #endregion

    #region Plugin Methods
    public Guid AddPlugin(Plugin plugin, List<PluginData> pluginData) {
      // todo
      return Guid.NewGuid();
    }

    public IEnumerable<Plugin> GetPlugins() {
      if (plugins == null)
        plugins = ReadPluginsFromServerCache();
      return plugins;              
    }

    public IEnumerable<PluginData> GetPluginDatas(List<Guid> pluginIds) {
      List<PluginData> pluginDatas = new List<PluginData>();

      foreach (Guid guid in pluginIds) {
        Plugin plugin = plugins.Find(p => p.Id == guid);
        if (plugin != null) {
          PluginDescription desc = pDescs.First(p => p.Name == plugin.Name && p.Version == plugin.Version);
          if (desc != null) {
            foreach (IPluginFile file in desc.Files) {
              PluginData data = new PluginData();
              data.PluginId = guid;
              data.Data = File.ReadAllBytes(file.Name);
              pluginDatas.Add(data);
            }
          }          
        }     
      }
      return pluginDatas;
    }
    #endregion

    #region Slave Methods
    public Guid AddSlave(Services.Hive.Common.DataTransfer.Slave slave) {
      throw new NotImplementedException();
    }

    public Guid AddSlaveGroup(SlaveGroup slaveGroup) {
      throw new NotImplementedException();
    }

    public IEnumerable<Services.Hive.Common.DataTransfer.Slave> GetSlaves() {
      throw new NotImplementedException();
    }

    public IEnumerable<SlaveGroup> GetSlaveGroups() {
      throw new NotImplementedException();
    }

    public void DeleteSlaveGroup(Guid clientGroupId) {
      throw new NotImplementedException();
    }

    public void AddResourceToGroup(Guid slaveGroupId, Resource resource) {
      throw new NotImplementedException();
    }

    public void RemoveResourceFromGroup(Guid slaveGroupId, Guid resourceId) {
      throw new NotImplementedException();
    }

    public void UpdateSlave(Services.Hive.Common.DataTransfer.Slave slave) {
      throw new NotImplementedException();
    }
    #endregion
  }
}
