﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Hive;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using System.Drawing;
using System.Threading;

namespace HeuristicLab.Clients.Hive.Slave.Tests {
  [Item("Mock Job", "Represents mocked Job which just SpinWaits.")]
  [StorableClass]
  public class MockJob : NamedItem, IJob {
    public virtual bool IsParallelizable {
      get { return false; }
    }

    [Storable]
    protected bool computeInParallel;
    public bool ComputeInParallel {
      get { return computeInParallel; }
      set {
        if (computeInParallel != value) {
          computeInParallel = value;
          OnComputeInParallelChanged();
        }
      }
    }

    [Storable]
    private int indexInParentOptimizerList = -1;
    public int IndexInParentOptimizerList {
      get { return indexInParentOptimizerList; }
      set { this.indexInParentOptimizerList = value; }
    }

    [Storable]
    private bool collectChildJobs;
    public bool CollectChildJobs {
      get { return collectChildJobs; }
      set { collectChildJobs = value; }
    }

    private int ms = 1000; //ms
    public MockJob(int ms) {
      this.ms = ms;
    }
    public MockJob() { }
    [StorableConstructor]
    protected MockJob(bool deserializing) { }
    protected MockJob(MockJob original, Cloner cloner) : base(original, cloner) {
      this.ComputeInParallel = original.ComputeInParallel;
      this.IndexInParentOptimizerList = original.IndexInParentOptimizerList;
      this.CollectChildJobs = original.CollectChildJobs;
      this.ExecutionTime = original.executionTime;
      this.ExecutionState = original.executionState;
      this.ms = original.ms;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MockJob(this, cloner);
    }

    [Storable]
    private ExecutionState executionState;
    public virtual ExecutionState ExecutionState {
      get { return executionState; }
      set {
        if (executionState != value) {
          this.executionState = value;
          OnExecutionStateChanged();
        }
      }
    }

    [Storable]
    private TimeSpan executionTime;
    public TimeSpan ExecutionTime {
      get { return executionTime; }
      set {
        if (executionTime != value) {
          executionTime = value;
          OnExecutionTimeChanged();
        }
      }
    }

    public virtual void Prepare() {
      this.ExecutionState = Core.ExecutionState.Prepared;
    }

    public virtual void Start() {
      DateTime start = DateTime.Now;
      do {
        Thread.SpinWait(1000);
        this.ExecutionTime = DateTime.Now - start;
      } while (ExecutionTime.TotalMilliseconds < ms);
      Stop();
      OnJobStopped();
    }

    public virtual void Pause() {
      this.ExecutionState = Core.ExecutionState.Paused;
    }

    public virtual void Stop() {
      ExecutionState = Core.ExecutionState.Stopped;
    }

    public virtual void Resume(IEnumerable<IJob> childJobs) {
      ExecutionState = Core.ExecutionState.Stopped;
    }

    #region Events
    public event EventHandler<EventArgs<IJob>> NewChildJob;
    protected virtual void OnNewChildJob(IJob job) {
      EventHandler<EventArgs<IJob>> handler = NewChildJob;
      if (handler != null) handler(this, new EventArgs<IJob>(job));
    }

    public event EventHandler WaitForChildJobs;
    protected virtual void OnWaitForChildJobs() {
      EventHandler handler = WaitForChildJobs;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler DeleteChildJobs;
    protected virtual void OnDeleteChildJobs() {
      EventHandler handler = DeleteChildJobs;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler ComputeInParallelChanged;
    protected virtual void OnComputeInParallelChanged() {
      EventHandler handler = ComputeInParallelChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler ExecutionTimeChanged;
    protected virtual void OnExecutionTimeChanged() {
      EventHandler handler = ExecutionTimeChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler ExecutionStateChanged;
    protected virtual void OnExecutionStateChanged() {
      EventHandler handler = ExecutionStateChanged;
      if (handler != null) handler(this, EventArgs.Empty);
    }
                                                      
    public event EventHandler JobFailed;
    protected virtual void OnJobFailed() {
      EventHandler handler = JobFailed;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    public event EventHandler JobStopped;
    protected virtual void OnJobStopped() {
      EventHandler handler = JobStopped;
      if (handler != null) handler(this, EventArgs.Empty);
    }

    #endregion
  }
}
