﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.IO;
using HeuristicLab.Services.Hive.Common.DataTransfer;
using System.Net.Security;

namespace HeuristicLab.Services.Hive.Common.ServiceContracts {

  [ServiceContract(ProtectionLevel = ProtectionLevel.EncryptAndSign)]
  public interface IHiveService {

    #region Job Methods
    [OperationContract]
    Guid AddJob(Job job, JobData jobData);

    [OperationContract]
    Guid AddChildJob(Guid parentJobId, Job job, JobData jobData);

    [OperationContract] // formerly GetJobById
    Job GetJob(Guid jobId);

    [OperationContract] // formerly GetAllJobs
    IEnumerable<Job> GetJobs();

    [OperationContract] // formerly GetJobResults
    IEnumerable<LightweightJob> GetLightweightJobs(IEnumerable<Guid> jobIds);

    [OperationContract] // formerly GetChildJobResults
    IEnumerable<LightweightJob> GetLightweightChildJobs(Guid? parentJobId, bool recursive, bool includeParent);

    [OperationContract] // formerly GetLastSerializedResult
    JobData GetJobData(Guid jobId);

    [OperationContract]
    void UpdateJob(Job jobDto, JobData jobDataDto);

    [OperationContract]
    void DeleteChildJobs(Guid parentJobId);

    [OperationContract] // new method: appropriate job is choosen and set to 'calculating'. the slave is responsible for requesting the jobData. Server should wait some timeout until he redistributes the job
    Job AquireJob(Guid slaveId);

    [OperationContract]
    PluginData GetConfigurationFile();

    #endregion

    #region Job Control Methods
    [OperationContract]
    void StopJob(Guid jobId);

    [OperationContract]
    void PauseJob(Guid jobId);
    #endregion

    #region HiveExperiment Methods
    [OperationContract]
    HiveExperiment GetHiveExperiment(Guid id);

    /// <summary>
    /// Returns all experiments for the current user
    /// </summary>
    [OperationContract]
    IEnumerable<HiveExperiment> GetHiveExperiments();

    [OperationContract]
    Guid AddHiveExperiment(HiveExperiment hiveExperimentDto);

    [OperationContract]
    void UpdateHiveExperiment(HiveExperiment hiveExperimentDto);

    [OperationContract]
    void DeleteHiveExperiment(Guid hiveExperimentId);
    #endregion

    #region Login Methods
    [OperationContract]
    void Hello(Guid slaveId, string name, int cores, int memory);

    [OperationContract]
    void GoodBye();
    #endregion

    #region Heartbeat Methods
    [OperationContract]
    List<MessageContainer> Heartbeat(Heartbeat heartbeat);
    #endregion

    #region Plugin Methods
    [OperationContract]
    Guid AddPlugin(Plugin plugin, List<PluginData> pluginData);
    [OperationContract]
    IEnumerable<Plugin> GetPlugins();
    [OperationContract]
    IEnumerable<PluginData> GetPluginDatas(List<Guid> pluginIds);
    #endregion
    
    #region Slave Methods
    [OperationContract]
    Guid AddSlave(Slave slave);

    [OperationContract]
    Guid AddSlaveGroup(SlaveGroup slaveGroup);

    [OperationContract]
    IEnumerable<Slave> GetSlaves();
    
    [OperationContract]
    [ServiceKnownType(typeof(Resource))]
    [ServiceKnownType(typeof(Slave))]
    [ServiceKnownType(typeof(SlaveGroup))]
    IEnumerable<SlaveGroup> GetSlaveGroups();

    [OperationContract]
    void DeleteSlaveGroup(Guid clientGroupId);

    [OperationContract]
    [ServiceKnownType(typeof(Resource))]
    [ServiceKnownType(typeof(Slave))]
    [ServiceKnownType(typeof(SlaveGroup))]
    void AddResourceToGroup(Guid slaveGroupId, Resource resource);

    [OperationContract]
    void RemoveResourceFromGroup(Guid slaveGroupId, Guid resourceId);

    [OperationContract]
    void UpdateSlave(Slave slave);
    #endregion

  }
}
